;   intr_.asm
;
;     This file provides run time installable ISR capability through the use of
;     the intr_jump_table which is defined in intr.c.  This file provides the 
;     ISFPs (Interrupt Service Fetch Packets) for the IST (Interrupt Service
;     Table).
;     
;-------------------------------------------------------------------------------
; REVISION HISTORY
;
; 11MAY98 The Interrupt Service Table setup was redone to allow for large memory
;         model. 
;
; 13MAY98 Unhooked interrupt vectors branch to default interrupt handler traps
;         instead of ignoring interrupt.
;
; 13MAY98 Added optional ".res" section to support non-zero interrupt
;         service table base address.
;
;-------------------------------------------------------------------------------
            .ref          _c_int00     ;reset ISR
            .ref          _isr_jump_table
            .global       _istb        ; interrupt service table base

            .global       _RESET, _NMI,   _RESV1, _RESV2
            .global       _INT4 , _INT5 , _INT6 , _INT7
            .global       _INT8,  _INT9,  _INT10, _INT11
            .global       _INT12, _INT13, _INT14, _INT15
            .global       _unexp_int04, _unexp_int05
            .global       _unexp_int06, _unexp_int07
            .global       _unexp_int08, _unexp_int09
            .global       _unexp_int10, _unexp_int11
            .global       _unexp_int12, _unexp_int13
            .global       _unexp_int14, _unexp_int15

;-------------------------------------------------------------------------------
; NOTE: The following ".res" section must be uncommented if the interrupt
;       service table base address (ISTB in the ISTP register) is non-zero
;       to provide the required reset vector at address 0.  If the ".res"
;       section is uncommented, then the linker command file must include
;       it and map it to address 0.
;-------------------------------------------------------------------------------
;           .sect         ".res"       ;.res section must be linked @ address 0
;           mvkl        _c_int00,b0
;           mvkh        _c_int00,b0
;           b     .s2   b0
;           nop         5
;           nop
;           nop
;           nop
;           nop
;-------------------------------------------------------------------------------

            .sect         ".vec"

           .align 1024 ; The ISTB field occupies bits 31 -- 10 in ISTP, so bits
                       ;    9 -- 0 of _istb should be 0.

_istb:
_RESET:     mvkl        _c_int00,b0
            mvkh        _c_int00,b0
            b     .s2   b0
            nop         5
            nop
            nop
            nop
            nop

_NMI:       .asg        1, vec
            stw   .d2   b0, *--b15
      ||    mvkl        (_isr_jump_table + vec * 4), b0
            mvkh        (_isr_jump_table + vec * 4), b0
            ldw   .d2   *b0, b0
            nop         4
            b     .s2   b0
      ||    ldw   .d2   *b15++, b0
            nop         5


_RESV1:     b     .s2   _RESV1
            nop
            nop
            nop
            nop
            nop
            nop
            nop



_RESV2:     b     .s2   _RESV2
            nop
            nop
            nop
            nop
            nop
            nop
            nop
     

_INT4:      .asg        4, vec
            stw   .d2   b0, *--b15
     ||     mvkl        (_isr_jump_table + vec * 4), b0
            mvkh        (_isr_jump_table + vec * 4), b0
            ldw   .d2   *b0, b0
            nop         4
            b     .s2   b0
     ||     ldw   .d2   *b15++, b0
            nop         5



_INT5:      .asg        5, vec
            stw   .d2   b0, *--b15
     ||     mvkl        (_isr_jump_table + vec * 4), b0
            mvkh        (_isr_jump_table + vec * 4), b0
            ldw   .d2   *b0, b0
            nop         4
            b     .s2   b0
     ||     ldw   .d2   *b15++, b0
            nop         5


_INT6:      .asg        6, vec
            stw   .d2   b0, *--b15
     ||     mvkl        (_isr_jump_table + vec * 4), b0
            mvkh        (_isr_jump_table + vec * 4), b0
            ldw   .d2   *b0, b0
            nop         4
            b     .s2   b0
     ||     ldw   .d2   *b15++, b0
            nop         5



_INT7:      .asg        7, vec
            stw   .d2   b0, *--b15
     ||     mvkl        (_isr_jump_table + vec * 4), b0
            mvkh        (_isr_jump_table + vec * 4), b0
            ldw   .d2   *b0, b0
            nop         4
            b     .s2   b0
     ||     ldw   .d2   *b15++, b0
            nop         5


_INT8:      .asg        8, vec
            stw   .d2   b0, *--b15
     ||     mvkl        (_isr_jump_table + vec * 4), b0
            mvkh        (_isr_jump_table + vec * 4), b0
            ldw   .d2   *b0, b0
            nop         4
            b     .s2   b0
     ||     ldw   .d2   *b15++, b0
            nop         5


_INT9:      .asg        9, vec
            stw   .d2   b0, *--b15
     ||     mvkl        (_isr_jump_table + vec * 4), b0
            mvkh        (_isr_jump_table + vec * 4), b0
            ldw   .d2   *b0, b0
            nop         4
            b     .s2   b0
     ||     ldw   .d2   *b15++, b0
            nop         5


_INT10:     .asg        10, vec
            stw   .d2   b0, *--b15
     ||     mvkl        (_isr_jump_table + vec * 4), b0
            mvkh        (_isr_jump_table + vec * 4), b0
            ldw   .d2   *b0, b0
            nop         4
            b     .s2   b0
     ||     ldw   .d2   *b15++, b0
            nop         5


_INT11:     .asg        11, vec
            stw   .d2   b0, *--b15
     ||     mvkl        (_isr_jump_table + vec * 4), b0
            mvkh        (_isr_jump_table + vec * 4), b0
            ldw   .d2   *b0, b0
            nop         4
            b     .s2   b0
     ||     ldw   .d2   *b15++, b0
            nop         5


_INT12:     .asg        12, vec
            stw   .d2   b0, *--b15
     ||     mvkl        (_isr_jump_table + vec * 4), b0
            mvkh        (_isr_jump_table + vec * 4), b0
            ldw   .d2   *b0, b0
            nop         4
            b     .s2   b0
     ||     ldw   .d2   *b15++, b0
            nop         5


_INT13:     .asg        13, vec
            stw   .d2   b0, *--b15
     ||     mvkl        (_isr_jump_table + vec * 4), b0
            mvkh        (_isr_jump_table + vec * 4), b0
            ldw   .d2   *b0, b0
            nop         4
            b     .s2   b0
     ||     ldw   .d2   *b15++, b0
            nop         5


_INT14:     .asg        14, vec
            stw   .d2   b0, *--b15
     ||     mvkl        (_isr_jump_table + vec * 4), b0
            mvkh        (_isr_jump_table + vec * 4), b0
            ldw   .d2   *b0, b0
            nop         4
            b     .s2   b0
     ||     ldw   .d2   *b15++, b0
            nop         5


_INT15:     .asg        15, vec
            stw   .d2   b0, *--b15
     ||     mvkl        (_isr_jump_table + vec * 4), b0
            mvkh        (_isr_jump_table + vec * 4), b0
            ldw   .d2   *b0, b0
            nop         4
            b     .s2   b0
     ||     ldw   .d2   *b15++, b0
            nop         5


; Program flow enters one of the following labels when an enabled interrupt has 
; no ISR hooked to it. To avoid this, use intr_hook(ISR, CPU_INTXX) function.

_unexp_int04:
            b     .s2   _unexp_int04
            nop         5
      
_unexp_int05:
            b     .s2   _unexp_int05
            nop         5

_unexp_int06:
            b     .s2   _unexp_int06
            nop         5

_unexp_int07:
            b     .s2   _unexp_int07
            nop         5

_unexp_int08:
            b     .s2   _unexp_int08
            nop         5

_unexp_int09:
            b     .s2   _unexp_int09
            nop         5

_unexp_int10:
            b     .s2   _unexp_int10
            nop         5

_unexp_int11:
            b     .s2   _unexp_int11
            nop         5

_unexp_int12:
            b     .s2   _unexp_int12
            nop         5

_unexp_int13:
            b     .s2   _unexp_int13
            nop         5

_unexp_int14:
            b     .s2   _unexp_int14
            nop         5

_unexp_int15:
            b     .s2   _unexp_int15
            nop         5

