;*************************************************************************
; Interrupt Vector Assignments for CCS v5.1.
;*************************************************************************
	.title "C6x Interrupt Vectors "
	.tab	4
	
SP			.set	B15			; Redefine for convenience


;************************************************************************
;	Interrupt Service Table (IST)
; 
; This is a table of Interrupt Service Fetch Packets (ISFP) which
; contain code for servicing the interrupts.
; This serves as a multi-purpose example for the C6x family.
; Consult the TI DSP Reference manual to determine which interrupts 
; apply to a given target and/or application.
; The IST consists of 16 consecutive fetch ISFPs.
* Each ISFP contains 8 32-bit instructions, or 32bytes of code.
;************************************************************************

    .ref   _isr_jump_table      ; Defined in DSKintr.c

	.sect	".intvecs"
	.align	32*8*4				; must be aligned on 256 word boundary
	.def RESET_V, _istb
_istb:
RESET_V:			; Reset
	.ref	_c_int00			; program reset address
	MVKL 	_c_int00, B3
	MVKH	_c_int00, B3
	B		B3
	MVC 	PCE1, B0			; get address of interrupt vectors
	MVC 	B0, ISTP			; set table to point here
	NOP 	
	NOP 	
	NOP 	
	
	.align	32
	.def NMI_V, _NMI
_NMI:
NMI_V:				; Non-maskable interrupt Vector
      .asg        1, vec
            stw   .d2   b0, *--b15
      ||    mvkl        (_isr_jump_table + vec * 4), b0
            mvkh        (_isr_jump_table + vec * 4), b0
            ldw   .d2   *b0, b0
            nop         4
            b     .s2   b0
      ||    ldw   .d2   *b15++, b0
            nop         5
	
	.align	32
	.def AINT_V, _RESV1
_RESV1:
AINT_V:			; Analysis Interrupt Vector (reserved)
	B		$
	NOP 	
	NOP 	
	NOP 	
	NOP 	
	NOP 	
	NOP 	
	NOP 	
	
	.align	32
	.def MSGINT_V, _RESV2
_RESV2:
MSGINT_V:	; Reserved by TI
	B		$
	NOP 	
	NOP 	
	NOP 	
	NOP 	
	NOP 	
	NOP 	
	NOP 	

	.align	32
	.def INT4_V
INT4_V:				; Maskable Interrupt #4
     .asg        4, vec
            stw   .d2   b0, *--b15
     ||     mvkl        (_isr_jump_table + vec * 4), b0
            mvkh        (_isr_jump_table + vec * 4), b0
            ldw   .d2   *b0, b0
            nop         4
            b     .s2   b0
     ||     ldw   .d2   *b15++, b0
            nop         5
	
	.align	32
	.def INT5_V
INT5_V:				; Maskable Interrupt #5
     .asg        5, vec
            stw   .d2   b0, *--b15
     ||     mvkl        (_isr_jump_table + vec * 4), b0
            mvkh        (_isr_jump_table + vec * 4), b0
            ldw   .d2   *b0, b0
            nop         4
            b     .s2   b0
     ||     ldw   .d2   *b15++, b0
            nop         5
	
	.align	32
	.def INT6_V
INT6_V:				; Maskable Interrupt #6
     .asg        6, vec
            stw   .d2   b0, *--b15
     ||     mvkl        (_isr_jump_table + vec * 4), b0
            mvkh        (_isr_jump_table + vec * 4), b0
            ldw   .d2   *b0, b0
            nop         4
            b     .s2   b0
     ||     ldw   .d2   *b15++, b0
            nop         5
	
	.align	32
	.def INT7_V
INT7_V:				; Maskable Interrupt #7
     .asg        7, vec
            stw   .d2   b0, *--b15
     ||     mvkl        (_isr_jump_table + vec * 4), b0
            mvkh        (_isr_jump_table + vec * 4), b0
            ldw   .d2   *b0, b0
            nop         4
            b     .s2   b0
     ||     ldw   .d2   *b15++, b0
            nop         5

	.align	32
	.def INT8_V
INT8_V:				; Maskable Interrupt #8
     .asg        8, vec
            stw   .d2   b0, *--b15
     ||     mvkl        (_isr_jump_table + vec * 4), b0
            mvkh        (_isr_jump_table + vec * 4), b0
            ldw   .d2   *b0, b0
            nop         4
            b     .s2   b0
     ||     ldw   .d2   *b15++, b0
            nop         5

	.align	32
	.def INT9_V
INT9_V:				; Maskable Interrupt #9
     .asg        9, vec
            stw   .d2   b0, *--b15
     ||     mvkl        (_isr_jump_table + vec * 4), b0
            mvkh        (_isr_jump_table + vec * 4), b0
            ldw   .d2   *b0, b0
            nop         4
            b     .s2   b0
     ||     ldw   .d2   *b15++, b0
            nop         5

	.align	32
	.def INT10_V
INT10_V: 			; Maskable Interrupt #10
     .asg        10, vec
            stw   .d2   b0, *--b15
     ||     mvkl        (_isr_jump_table + vec * 4), b0
            mvkh        (_isr_jump_table + vec * 4), b0
            ldw   .d2   *b0, b0
            nop         4
            b     .s2   b0
     ||     ldw   .d2   *b15++, b0
            nop         5

	.align	32
	.def INT11_V
INT11_V: 			; Maskable Interrupt #11
	.if $isdefed("HSRTDX")
	.def RTDX_XMT_INTV
RTDX_XMT_INTV:		; HSRTDX Xmit Complete
	.ref	_HSRTDX_xmt_isr
	STW		B0,*SP--[2]				; save B0 temporarily
||	MVKL 	_HSRTDX_xmt_isr, B0		; load ISR address
	MVKH	_HSRTDX_xmt_isr, B0
	B		B0						; execute ISR;
	LDW		*++SP[2],B0			; restore B0
	NOP 	2
	NOP
	NOP
	.else
     .asg        11, vec
            stw   .d2   b0, *--b15
     ||     mvkl        (_isr_jump_table + vec * 4), b0
            mvkh        (_isr_jump_table + vec * 4), b0
            ldw   .d2   *b0, b0
            nop         4
            b     .s2   b0
     ||     ldw   .d2   *b15++, b0
            nop         5
     .endif
	
	.align	32
	.def INT12_V
INT12_V: 			; Maskable Interrupt #12
	.if $isdefed("HSRTDX")
	.def RTDX_REC_INTV
RTDX_REC_INTV:		; HSRTDX Rcvd Complete
	.ref	_HSRTDX_rec_isr
	STW		B0,*SP--[2]			; save B0 temporarily
||	MVKL 	_HSRTDX_rec_isr, B0		; load ISR address
	MVKH	_HSRTDX_rec_isr, B0
	B		B0						; execute ISR;
	LDW		*++SP[2],B0			; restore B0
	NOP 	2
	NOP
	NOP
	.else
     .asg        12, vec
            stw   .d2   b0, *--b15
     ||     mvkl        (_isr_jump_table + vec * 4), b0
            mvkh        (_isr_jump_table + vec * 4), b0
            ldw   .d2   *b0, b0
            nop         4
            b     .s2   b0
     ||     ldw   .d2   *b15++, b0
            nop         5
     .endif
	
	.align	32
	.def INT13_V
INT13_V: 			; Maskable Interrupt #13

     .asg        13, vec
            stw   .d2   b0, *--b15
     ||     mvkl        (_isr_jump_table + vec * 4), b0
            mvkh        (_isr_jump_table + vec * 4), b0
            ldw   .d2   *b0, b0
            nop         4
            b     .s2   b0
     ||     ldw   .d2   *b15++, b0
            nop         5
	
	.align	32
	.def INT14_V
INT14_V: 			; Maskable Interrupt #14
     .asg        14, vec
            stw   .d2   b0, *--b15
     ||     mvkl        (_isr_jump_table + vec * 4), b0
            mvkh        (_isr_jump_table + vec * 4), b0
            ldw   .d2   *b0, b0
            nop         4
            b     .s2   b0
     ||     ldw   .d2   *b15++, b0
            nop         5
	
	.align	32
	.def INT15_V
INT15_V: 			; Maskable Interrupt #15
     .asg        15, vec
            stw   .d2   b0, *--b15
     ||     mvkl        (_isr_jump_table + vec * 4), b0
            mvkh        (_isr_jump_table + vec * 4), b0
            ldw   .d2   *b0, b0
            nop         4
            b     .s2   b0
     ||     ldw   .d2   *b15++, b0
            nop         5
	

; Program flow enters one of the following labels when an enabled interrupt has 
; no ISR hooked to it. To avoid this, use intr_hook(ISR, CPU_INTXX) function.

    .def _unexp_int04
_unexp_int04:
            b     .s2   _unexp_int04
            nop         5

    .def _unexp_int05      
_unexp_int05:
            b     .s2   _unexp_int05
            nop         5

    .def _unexp_int06
_unexp_int06:
            b     .s2   _unexp_int06
            nop         5

    .def _unexp_int07
_unexp_int07:
            b     .s2   _unexp_int07
            nop         5

    .def _unexp_int08
_unexp_int08:
            b     .s2   _unexp_int08
            nop         5

    .def _unexp_int09
_unexp_int09:
            b     .s2   _unexp_int09
            nop         5

    .def _unexp_int10
_unexp_int10:
            b     .s2   _unexp_int10
            nop         5

    .def _unexp_int11
_unexp_int11:
            b     .s2   _unexp_int11
            nop         5


    .def _unexp_int12
_unexp_int12:
            b     .s2   _unexp_int12
            nop         5

    .def _unexp_int13
_unexp_int13:
            b     .s2   _unexp_int13
            nop         5

    .def _unexp_int14
_unexp_int14:
            b     .s2   _unexp_int14
            nop         5

    .def _unexp_int15
_unexp_int15:
            b     .s2   _unexp_int15
            nop         5

	.end
